/**
 * @file 处理组件相关
 * @author wangyisheng@baidu.com (wangyisheng)
 */

// Store name of loaded component scripts
// e.g. ['tree', 'hello']
let loadedComponents = [];

// Store map from components name to src
// e.g. {'tree': 'some.cdn.com/mip-components/mip-tree.js'}
let componentsSrcMap = Object.create(null);
// Get names of components which need to be loaded
export function getNewComponents(diffComponents) {
    let newComponents = [];

    diffComponents.forEach(diffCom => {
        if (loadedComponents.indexOf(diffCom) === -1) {
            newComponents.push(diffCom);
        }
    });

    return newComponents;
}

// Store names of components which has already been loaded
// Explictly used in first loading
export function addLoadedComponents(newComponents) {
    loadedComponents = loadedComponents.concat(newComponents);
}

// Load components by names
// Some components need additional template. Find them and append it. See https://github.com/mip-project/mip/issues/14
export async function loadScripts(components, targetHTML) {
    function loadScript(src) {
        return new Promise((resolve, reject) => {
            let script = document.createElement('script');
            script.type = 'text/javascript';
            script.onload = resolve;
            script.onerror = reject;
            script.src = src;
            document.body.appendChild(script);
        });
    }

    function loadTemplateScript(componentName, content) {
        let templateScript = document.createElement('script');
        templateScript.type = 'text/x-template';
        templateScript.id = `mip-${componentName}-template`;
        templateScript.innerHTML = content;
        document.body.appendChild(templateScript);
    }

    await Promise.all(components.map(com => {
        // find template script for some components
        let reg = new RegExp('<script[\\s\\S]*?id=[\'"]?mip-' + com + '-template[\\s\\S]*?>([\\s\\S]+?)</script>');
        let match = targetHTML.match(reg);
        if (match) {
            loadTemplateScript(com, match[1]);
        }

        return loadScript(componentsSrcMap[com])
    }));

    addLoadedComponents(components);
}
