/**
 * @file define getter functions
 * @author wangyisheng@baidu.com (wangyisheng)
 */

const MIP_VIEW_ID = 'mip-router__view';

export function getMIPTitle(rawContent) {
    let title;

    if (!rawContent) {
        let titleDom = document.querySelector('title');
        if (titleDom) {
            title = titleDom.innerHTML.trim();
        }
    }
    else {
        let match = rawContent.match(/<title>([\s\S]+)<\/title>/);
        if (match) {
            title = match[1];
        }
    }

    return title || '百度一下，你就知道';
}

export function getMIPData(rawContent) {
    let rawData;

    if (!rawContent) {
        let dataDom = document.querySelector('mip-data');
        if (!dataDom) {
            return;
        }

        rawData = dataDom.innerHTML.trim();
    }
    else {
        let match = rawContent.match(/<mip-data>([\s\S]+)<\/mip-data>/);
        if (!match) {
            return;
        }

        rawData = match[1].trim();
    }

    return JSON.parse(rawData);
}

export function getMIPContent(rawContent) {
    let rawResult;

    if (!rawContent) {
        let contentDom = document.querySelector('mip-content');

        if (!contentDom) {
            return '';
        }

        rawResult = contentDom.innerHTML.trim();
    }
    else {
        let match = rawContent.match(/<mip-content>([\s\S]+)<\/mip-content>/);

        if (!match) {
            return '';
        }

        rawResult = match[1].trim();
    }

    // Create a root node
    return `<div id="${MIP_VIEW_ID}">${rawResult}</div>`;
}

export function getMIPComponents(rawContent) {
    // Eliminate duplicate scripts
    let componentsSet = new Set();

    if (!rawContent) {
        document.querySelectorAll('script').forEach(script => {
            let src = script.getAttribute('src') || '';
            let match = src.match(/\/mip-(.+)\.js$/);

            // filter mip-router.js out
            // TODO delete me when mip-router.js is combined with mip.js
            if (match && match !== 'router') {
                componentsSet.add(match[1]);
                componentsSrcMap[match[1]] = src;
            }
        });
    }
    else {
        let match = rawContent.match(/<script[\s\S]*?>/g);

        if (match) {
            match.forEach(scriptAttrStr => {
                let innerMatch = scriptAttrStr.match(/src=['"]?(.+\/mip-(.+?)\.js)/);

                if (innerMatch && innerMatch[2] !== 'router') {
                    componentsSet.add(innerMatch[2]);
                    componentsSrcMap[innerMatch[2]] = innerMatch[1];
                }
            });
        }
    }

    let components = [];
    componentsSet.forEach(com => components.push(com));
    return components;
}
